/*
 *  Copyright (C) 2004 Jonas B. Jacobi <jonas.jacobi@web.de>   
 *  Partially copied from openwithpart
 */

#include <qpopupmenu.h>

#include <kiconloader.h>
#include <klocale.h>
#include <kgenericfactory.h>
#include <kfileitem.h>
#include <kaction.h>
#include <kmessagebox.h>
#include <kparts/part.h>
#include <ktexteditor/document.h>

#include <kdevcore.h>
#include <kdevmainwindow.h>
#include <kdevgenericfactory.h>
#include <partcontroller.h>
#include <urlutil.h>
#include <kaboutdata.h>

#include "previewdoxygen_part.h"

static const KAboutData data("kdevpreviewdoxygen", I18N_NOOP("Preview Doxygen"), "0.9");
typedef KDevGenericFactory<previewdoxygenPart> previewdoxygenFactory;
K_EXPORT_COMPONENT_FACTORY( libkdevpreviewdoxygen, previewdoxygenFactory( &data) )

previewdoxygenPart::previewdoxygenPart( QObject *parent, const char *name, const QStringList& )
		: KDevPlugin( "PreviewDoxygen", "PreviewDoxygen", parent, name ? name : "previewdoxygenPart" ) , m_tmpDir(){
	setInstance( previewdoxygenFactory::instance() );
	setXMLFile( "kdevpart_previewdoxygen.rc" );
	connect( core(), SIGNAL( contextMenu( QPopupMenu*, const Context * ) ),
	         this, SLOT( fillContextMenu( QPopupMenu*, const Context * ) ) );
	connect(&m_process, SIGNAL(processExited(KProcess*)), this, SLOT(slotProcessExited()));
	connect( partController(), SIGNAL(activePartChanged(KParts::Part*)), this, SLOT(slotActivePartChanged(KParts::Part* )));
	m_action = new KAction(i18n("Preview Doxygen Output"), "preview_doxygen", CTRL+SHIFT+Key_P, this, SLOT(slotRun()), actionCollection(), "show_preview_doxygen_output");
	m_action->setToolTip( i18n("Show a preview of the doxygen output of this file") );
    	m_action->setWhatsThis( i18n("<b>Preview Doxygen output</b><p>Runs Doxygen of the current file and shows the created index.html.") );
}

previewdoxygenPart::~previewdoxygenPart() {
	delete m_action;
}

void previewdoxygenPart::fillContextMenu( QPopupMenu * popup, const Context * context ) {
	//-> openwithpart.cpp
	if ( !context->hasType( Context::FileContext ) )
		return ;
	const FileContext *ctx = static_cast<const FileContext*>( context );
	if ( ctx->isDirectory() )
		return ;
	popup->insertSeparator();
	//<-
	popup->insertItem(i18n("Preview Doxygen Output"), this, SLOT(slotRun()));
}

void previewdoxygenPart::slotProcessExited( )
{
	partController()->showDocument(KURL(m_tmpDir.name()+"html/index.html"), "doxyprev");
}

void previewdoxygenPart::slotRun( )
{
	if (m_file.isNull())
		return;
	
	if (m_process.isRunning()) {
		if ( KMessageBox::warningYesNo(0, i18n("Previous Doxygen process is still running.\nDo you want to cancel that process?"))  == KMessageBox::Yes )
			m_process.kill();
		else 
			return;
	}
	
	m_process.clearArguments();
	
	m_process << "doxygen" << "-";
	m_process.start(KProcess::NotifyOnExit, KProcess::All);
	QString param = "GENERATE_LATEX=NO\nOUTPUT_DIRECTORY=" + m_tmpDir.name() + "\nINPUT=" + m_file;
	m_process.writeStdin(param.ascii(), param.length());
	m_process.closeStdin();
}

void previewdoxygenPart::slotActivePartChanged( KParts::Part * part )
{
	// -> idea from cppsupportpart.cpp
	KTextEditor::Document* doc = dynamic_cast<KTextEditor::Document*>(part);
	if (doc != 0) 
		m_file = doc->url().path();
	else
		m_file = QString::null;
	// <-
}

#include "previewdoxygen_part.moc"
